﻿using FliUtils.Mvvm;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FliProCameraLib
{
    [Serializable()]
    public class FliProToolSettings
    {

        //**************************************************************
        #region Public Contants

        #endregion Public Contants

        //**************************************************************
        #region Main Window Properties

        #endregion Main Window Properties

        //**************************************************************
        #region Camera Control Panel Properties

        /// <summary>
        /// Flag to signal the Cooler power control is Enabled (This is only Visible in Engineering mode).
        /// </summary>
        public bool CoolerDutyCycleIsEnabledOn
        {
            get { return _CoolerDutyCycleIsEnabledOn; }
            set { _CoolerDutyCycleIsEnabledOn = value; }
        }
        private bool _CoolerDutyCycleIsEnabledOn = false;

        /// <summary>
        /// The percentage of power the Cooler will use when enabled (This is only Visible in Engineering mode).
        /// </summary>
        public uint CoolerDutyCyclePower
        {
            get { return _CoolerDutyCyclePower; }
            set { _CoolerDutyCyclePower = value; }
        }
        private uint _CoolerDutyCyclePower = 0;

        /// <summary>
        /// Flag to signal the Fan is Enabled
        /// </summary>
        public bool CoolerTemperatureOn
        {
            get { return _CoolerTemperatureOn; }
            set { _CoolerTemperatureOn = value; }
        }
        private bool _CoolerTemperatureOn = true;

        public int DeviceType { get; set; }

        /// <summary>
        /// Flag to signal the Fan is Enabled
        /// </summary>
        public bool FanIsEnabledOn
        {
            get { return _FanIsEnabledOn; }
            set { _FanIsEnabledOn = value; }
        }
        private bool _FanIsEnabledOn = true;

        /// <summary>
        /// The percentage of power the Fan will use when enabled (This is only Visible in Engineering mode).
        /// </summary>
        public uint FanPower
        {
            get { return _FanPower; }
            set { _FanPower = value; }
        }
        private uint _FanPower = 85;

        /// <summary>
        /// Flag to signal the Heater is Enabled
        /// </summary>
        public bool HeaterEnabledOn
        {
            get { return _HeaterEnabledOn; }
            set { _HeaterEnabledOn = value; }
        }
        private bool _HeaterEnabledOn = false;

        /// <summary>
        /// The percentage of power the heater will use when enabled.
        /// </summary>
        public uint HeaterPower
        {
            get { return _HeaterPower; }
            set { _HeaterPower = value; }
        }
        private uint _HeaterPower = 0;

        /// <summary>
        /// The amount of time in milliseconds that the shutter close should be delayed.
        /// </summary>
        public int IlluminationOffDelay
        {
            get { return _IlluminationOffDelay; }
            set { _IlluminationOffDelay = value; }
        }
        private int _IlluminationOffDelay = 0;

        /// <summary>
        /// Flag to signal the shutter is open
        /// </summary>
        public bool IlluminationOn
        {
            get { return _IlluminationOn; }
            set { _IlluminationOn = value; }
        }
        private bool _IlluminationOn = false;

        /// <summary>
        /// The amount of time in milliseconds that the Illumination On should be delayed.
        /// </summary>
        public int IlluminationOnDelay
        {
            get { return _IlluminationOnDelay; }
            set { _IlluminationOnDelay = value; }
        }
        private int _IlluminationOnDelay = 0;

        /// <summary>
        /// Flag to signal the LED is On
        /// </summary>
        public bool LEDOn
        {
            get { return _LEDOn; }
            set { _LEDOn = value; }
        }
        private bool _LEDOn = false;

        /// <summary>
        /// Enables opening and closing the Shutter manually.
        /// </summary>
        public bool ManualShutterControlEnabled
        {
            get { return _ManualShutterControlEnabled; }
            set { _ManualShutterControlEnabled = value; }
        }
        protected bool _ManualShutterControlEnabled = false;

        /// <summary>
        /// The maximum number of frames that should be held in memory at one time per camera.
        /// </summary>
        public int MaximumFrameStorage
        {
            get { return _MaximumFrameStorage; }
            set { _MaximumFrameStorage = value; }
        }
        private int _MaximumFrameStorage = 10;

        /// <summary>
        /// Flag to signal that the user would like the temperature to continue being read during the exposure.
        /// </summary>
        public bool ReadDuringExposureOn
        {
            get { return _ReadDuringExposureOn; }
            set { _ReadDuringExposureOn = value; }
        }
        private bool _ReadDuringExposureOn = false;

        /// <summary>
        /// Flag to signal the shutter is closed
        /// </summary>
        public bool ShutterClose
        {
            get { return _ShutterClose; }
            set { _ShutterClose = value; }
        }
        private bool _ShutterClose = true;

        /// <summary>
        /// Flag to signal the shutter is open
        /// </summary>
        public bool ShutterOpen
        {
            get { return _ShutterOpen; }
            set { _ShutterOpen = value; }
        }
        private bool _ShutterOpen = false;

        /// <summary>
        /// The target temperature in °C that the cooler should achieve when enabled.
        /// </summary>
        public double TargetTemperature
        {
            get { return _TargetTemperature; }
            set { _TargetTemperature = value; }
        }
        private double _TargetTemperature = 25;

        /// <summary>
        /// The amount of time between temperature reads when Read During Exposure is active.
        /// </summary>
        public int TemperatureReadInterval
        {
            get { return _TemperatureReadInterval; }
            set { _TemperatureReadInterval = value; }
        }
        protected int _TemperatureReadInterval;

        #endregion Frame Panel Properties

        //**************************************************************
        #region Exposure Panel Properties

        /// <summary>
        /// The lenght of time of the exposure
        /// </summary>
        public int ExposureTime
        {
            get { return _ExposureTime; }
            set { _ExposureTime = value; }
        }
        private int _ExposureTime = 50;

        /// <summary>
        /// The units as defined by ExposureUnits_Enum by which the exposure time is measured
        /// </summary>
        public int ExposureUnits
        {
            get { return _ExposureUnits; }
            set { _ExposureUnits = value; }
        }
        private int _ExposureUnits = FliProCamera.ExposureUnits_Enum.units_mSecs.GetHashCode();

        /// <summary>
        /// The lenght of time of delay before the exposure
        /// </summary>
        public int ExposureDelayTime
        {
            get { return _ExposureDelayTime; }
            set { _ExposureDelayTime = value; }
        }
        private int _ExposureDelayTime = 0;

        /// <summary>
        /// The units as defined by ExposureUnits_Enum by which the exposure delay  time is measured
        /// </summary>
        public int ExposureDelayUnits
        {
            get { return _ExposureDelayUnits; }
            set { _ExposureDelayUnits = value; }
        }
        private int _ExposureDelayUnits = FliProCamera.ExposureUnits_Enum.units_mSecs.GetHashCode();

        /// <summary>
        /// The lenght of time of the Frame Interval which equals exposure plus delay
        /// </summary>
        public int ExposureFrameIntervalTime
        {
            get { return _ExposureFrameIntervalTime; }
            set { _ExposureFrameIntervalTime = value; }
        }
        private int _ExposureFrameIntervalTime = 50;

        /// <summary>
        /// The units as defined by ExposureUnits_Enum by which the Frame Interval time is measured
        /// </summary>
        public int ExposureFrameIntervalUnits
        {
            get { return _ExposureFrameIntervalUnits; }
            set { _ExposureFrameIntervalUnits = value; }
        }
        private int _ExposureFrameIntervalUnits = FliProCamera.ExposureUnits_Enum.units_mSecs.GetHashCode();

        /// <summary>
        /// Flag to signal the hardware to start the exposure from the current position in the clear process
        /// </summary>
        public bool Immediate
        {
            get { return _Immediate; }
            set { _Immediate = value; }
        }
        private bool _Immediate = false;

        /// <summary>
        /// Flag to signal which control the user has selected either Delay or Frame Interval
        /// </summary>
        public bool UseExposureDelay
        {
            get { return _UseExposureDelay; }
            set { _UseExposureDelay = value; }
        }
        private bool _UseExposureDelay = true;

        #endregion Exposure Panel Properties

        //**************************************************************
        #region File Name Setup Properties

        public int BinningOrder
        {
            get { return _BinningOrder; }
            set { _BinningOrder = value; }
        }
        private int _BinningOrder = 0;

        public int CameraModelOrder
        {
            get { return _CameraModelOrder; }
            set { _CameraModelOrder = value; }
        }
        private int _CameraModelOrder = 0;

        public int DateOrder
        {
            get { return _DateOrder; }
            set { _DateOrder = value; }
        }
        private int _DateOrder = 0;

        public int DiagRoutineOrder
        {
            get { return _DiagRoutineOrder; }
            set            { _DiagRoutineOrder = value; }
        }
        private int _DiagRoutineOrder = 0;

        public int ExposureTimeOrder
        {
            get { return _ExposureTimeOrder; }
            set { _ExposureTimeOrder = value; }
        }
        private int _ExposureTimeOrder = 0;

        public int FrameNumberOrder
        {
            get { return _FrameNumberOrder; }
            set { _FrameNumberOrder = value; }
        }
        private int _FrameNumberOrder = 0;

        public int GainOrder
        {
            get { return _GainOrder; }
            set { _GainOrder = value; }
        }
        private int _GainOrder = 0;

        public int GrabTypeOrder
        {
            get { return _GrabTypeOrder; }
            set { _GrabTypeOrder = value; }
        }
        private int _GrabTypeOrder = 0;

        public bool IsCheckedBinning
        {
            get { return _IsCheckedBinning; }
            set { _IsCheckedBinning = value; }
        }
        private bool _IsCheckedBinning = false;

        public bool IsCheckedCameraModel
        {
            get { return _IsCheckedCameraModel; }
            set { _IsCheckedCameraModel = value; }
        }
        private bool _IsCheckedCameraModel = false;

        public bool IsCheckedDate
        {
            get { return _IsCheckedDate; }
            set { _IsCheckedDate = value; }
        }
        private bool _IsCheckedDate = false;

        public bool IsCheckedExposureTime
        {
            get { return _IsCheckedExposureTime; }
            set { _IsCheckedExposureTime = value; }
        }
        private bool _IsCheckedExposureTime = false;

        public bool IsCheckedFrameNumber
        {
            get { return _IsCheckedFrameNumber; }
            set { _IsCheckedFrameNumber = value; }
        }
        private bool _IsCheckedFrameNumber = false;

        public bool IsCheckedGain
        {
            get { return _IsCheckedGain; }
            set { _IsCheckedGain = value; }
        }
        private bool _IsCheckedGain = false;
        
        public bool IsCheckedMean
        {
            get { return _IsCheckedMean; }
            set { _IsCheckedMean = value; }
        }
        private bool _IsCheckedMean = false;

        public bool IsCheckedMode
        {
            get { return _IsCheckedMode; }
            set { _IsCheckedMode = value; }
        }
        private bool _IsCheckedMode = false;

        public bool IsCheckedPreamble
        {
            get { return _IsCheckedPreamble; }
            set { _IsCheckedPreamble = value; }
        }
        private bool _IsCheckedPreamble = false;

        public bool IsCheckedSensorTemperature
        {
            get { return _IsCheckedSensorTemperature; }
            set { _IsCheckedSensorTemperature = value; }
        }
        private bool _IsCheckedSensorTemperature = false;

        public bool IsCheckedSerialNumber
        {
            get { return _IsCheckedSerialNumber; }
            set { _IsCheckedSerialNumber = value; }
        }
        private bool _IsCheckedSerialNumber = false;

        public int ModeOrder
        {
            get { return _ModeOrder; }
            set { _ModeOrder = value; }
        }
        private int _ModeOrder = 0;

        public string Preamble
        {
            get { return _Preamble; }
            set { _Preamble = value; }
        }
        private string _Preamble = String.Empty;

        public string RawFileName
        {
            get { return _RawFileName; }
            set { _RawFileName = value; }
        }
        private string _RawFileName = String.Empty;

        public int SensorTemperatureOrder
        {
            get { return _SensorTemperatureOrder; }
            set { _SensorTemperatureOrder = value; }
        }
        private int _SensorTemperatureOrder = 0;

        public int SerialNumberOrder
        {
            get { return _SerialNumberOrder; }
            set { _SerialNumberOrder = value; }
        }
        private int _SerialNumberOrder = 0;

        #endregion File Name Setup Properties

        //**************************************************************
        #region File Panel Properties

        /// <summary>
        /// The folder in which to save all the files.
        /// </summary>
        public string RawFilePath
        {
            get { return _RawFilePath; }
            set { _RawFilePath = value; }
        }
        private string _RawFilePath = String.Empty;

        #endregion Grab Panel Properties

        //**************************************************************
        #region Force Panel Properties

        /// <summary>
        /// The temperature range for auto training
        /// </summary>
        public double AutoForceRangeTemperature
        {
            get { return _AutoForceRangeTemperature; }
            set { _AutoForceRangeTemperature = value; }
        }
        private double _AutoForceRangeTemperature = 5;

        /// <summary>
        /// Flag to signal if auto training is on
        /// </summary>
        public bool IsCheckedAutoForceTraining
        {
            get { return _IsCheckedAutoForceTraining; }
            set { _IsCheckedAutoForceTraining = value; }
        }
        private bool _IsCheckedAutoForceTraining = true;

        #endregion Force Panel Properties

        //**************************************************************
        #region Frame Panel Properties

        /// <summary>
        /// Flag to signal the Image Data should be captured
        /// </summary>
        public bool ImageDataEnabled
        {
            get { return _ImageDataEnabled; }
            set { _ImageDataEnabled = value; }
        }
        private bool _ImageDataEnabled = true;

        /// <summary>
        /// The height of the target image Region Of Interest
        /// </summary>
        public int ImageHeight
        {
            get { return _ImageHeight; }
            set { _ImageHeight = value; }
        }
        private int _ImageHeight = 1;

        /// <summary>
        /// The X Origin of the image Region Of Interest
        /// </summary>
        public int ImageUpperLeftX
        {
            get { return _ImageUpperLeftX; }
            set { _ImageUpperLeftX = value; }
        }
        private int _ImageUpperLeftX = 0;

        /// <summary>
        /// The Y Origin of the image Region Of Interest
        /// </summary>
        public int ImageUpperLeftY
        {
            get { return _ImageUpperLeftY; }
            set { _ImageUpperLeftY = value; }
        }
        private int _ImageUpperLeftY = 0;

        /// <summary>
        /// The width of the target image Region Of Interest
        /// </summary>
        public int ImageWidth
        {
            get { return _ImageWidth; }
            set { _ImageWidth = value; }
        }
        private int _ImageWidth = 1;

        #endregion Frame Panel Properties

        //**************************************************************
        #region Grab Panel Properties

        /// <summary>
        /// FliFrameCount used to control which type of Grab will execute on the Grab Panel
        /// </summary>
        public int FrameCountType
        {
            get { return _FrameCountType; }
            set { _FrameCountType = value; }
        }
        private int _FrameCountType = FliPro.FliFrameCount.SINGLE.GetHashCode();

        /// <summary>
        /// FliFrameType used to control which type of Grab will execute on the Grab Panel
        /// </summary>
        public int FrameType
        {
            get { return _FrameType; }
            set { _FrameType = value; }
        }
        private int _FrameType = FliPro.FliFrameType.NORMAL.GetHashCode();

        #endregion Grab Panel Properties

        //**************************************************************
        #region Merge Panel Properties

        /// <summary>
        /// The ADUs per Electron which is calculated from the High and Low Gain images (or entered by the user if overridden)
        /// </summary>
        public double ADUsPerElectron
        {
            get { return _ADUsPerElectron; }
            set { _ADUsPerElectron = value; }
        }
        private double _ADUsPerElectron = 0;

        /// <summary>
        /// The Gain Ratio which is calculated from the High and Low Gain images (or entered by the user if overridden).
        /// More specifically, the factor to be multiplied with the Low Gain pixels to give them the same slope as the High Gain image.
        /// (This is only Visible in Engineering mode)
        /// </summary>
        public double GainRatio
        {
            get { return _GainRatio; }
            set { _GainRatio = value; }
        }
        private double _GainRatio = 0;

        /// <summary>
        /// The High Gain pixel value that signals where the Merge should calculate the new pixel from the Low Gain images.
        /// </summary>
        public int HighLowThreshold
        {
            get { return _HighLowThreshold; }
            set { _HighLowThreshold = value; }
        }
        private int _HighLowThreshold = 3800;

        /// <summary>
        /// The Slope Offset which is calculated from the High and Low Gain images (or entered by the user if overridden).
        /// This is the value to be added to the adjusted Low Gain pixels to align the Low Gain Line with the High Gain Line.
        /// </summary>
        public int LineOffset
        {
            get { return _LineOffset; }
            set { _LineOffset = value; }
        }
        private int _LineOffset = 0;

        #endregion Merge Panel Properties

        //**************************************************************
        #region Sensor Panel Properties

        /// <summary>
        /// Flag to signal to use the Horizontal Binning value in both directions
        /// </summary>
        public bool BinningSquared
        {
            get { return _BinningSquared; }
            set { _BinningSquared = value; }
        }
        private bool _BinningSquared = false;

        /// <summary>
        /// The Black Level High of the desired exposure the Sensor
        /// </summary>
        public uint BlackLevelHigh
        {
            get { return _BlackLevelHigh; }
            set { _BlackLevelHigh = value; }
        }
        private uint _BlackLevelHigh = 15800;

        /// <summary>
        /// The Black Level Low of the desired exposure the Sensor
        /// </summary>
        public uint BlackLevelLow
        {
            get { return _BlackLevelLow; }
            set { _BlackLevelLow = value; }
        }
        private uint _BlackLevelLow = 15800;

        /// <summary>
        /// The Black Sun High of the desired exposure the Sensor
        /// </summary>
        public uint BlackSunHigh
        {
            get { return _BlackSunHigh; }
            set { _BlackSunHigh = value; }
        }
        private uint _BlackSunHigh = 28;

        /// <summary>
        /// The Black Sun Low of the desired exposure the Sensor
        /// </summary>
        public uint BlackSunLow
        {
            get { return _BlackSunLow; }
            set { _BlackSunLow = value; }
        }
        private uint _BlackSunLow = 28;

        /// <summary>
        /// The Horizontal Binning to be used
        /// </summary>
        public uint HBin
        {
            get { return _HBin; }
            set { _HBin = value; }
        }
        private uint _HBin = 1;

        /// <summary>
        /// The active index for the High Gain image of the list of Gains available in the Sensor
        /// </summary>
        public int HighGainIndex
        {
            get { return _HighGainIndex; }
            set { _HighGainIndex = value; }
        }
        private int _HighGainIndex = 0;

        /// <summary>
        /// Flag to signal to invert in the Horizontal Scan Direction (ie. read the pixels in reverse)
        /// </summary>
        public bool HorizontalScanDirInverted
        {
            get { return _HorizontalScanDirInverted; }
            set { _HorizontalScanDirInverted = value; }
        }
        private bool _HorizontalScanDirInverted = false;

        /// <summary>
        /// The active index for the Low Gain image of the list of Gains available in the Sensor
        /// </summary>
        public int LowGainIndex
        {
            get { return _LowGainIndex; }
            set { _LowGainIndex = value; }
        }
        private int _LowGainIndex = 0;

        /// <summary>
        /// The active index of the list of Modes available in the Sensor
        /// </summary>
        public int Mode
        {
            get { return _Mode; }
            set { _Mode = value; }
        }
        private int _Mode = 0;

        /// <summary>
        /// The Pixel Depth of the desired exposure the Sensor
        /// </summary>
        public uint PixelDepth
        {
            get { return _PixelDepth; }
            set { _PixelDepth = value; }
        }
        private uint _PixelDepth = 12;

        /// <summary>
        /// Flag to signal to use software binning
        /// </summary>
        public bool UseSoftwareBinning
        {
            get { return _UseSoftwareBinning; }
            set { _UseSoftwareBinning = value; }
        }
        private bool _UseSoftwareBinning = false;

        /// <summary>
        /// The Vertical Binning to be used
        /// </summary>
        public uint VBin
        {
            get { return _VBin; }
            set { _VBin = value; }
        }
        private uint _VBin = 1;

        /// <summary>
        /// Flag to signal to invert in the Vertical Scan Direction (ie. read the pixels in reverse)
        /// </summary>
        public bool VerticalScanDirInverted
        {
            get { return _VerticalScanDirInverted; }
            set { _VerticalScanDirInverted = value; }
        }
        private bool _VerticalScanDirInverted = false;

        #endregion Sensor Panel Properties

        //**************************************************************
        #region Window Location Properties

        /// <summary>
        /// The Height of the Main Window
        /// </summary>
        public int MainWindowHeight
        {
            get { return _MainWindowHeight; }
            set { _MainWindowHeight = value; }
        }
        private int _MainWindowHeight = 800;

        /// <summary>
        /// The Left of the Main Window
        /// </summary>
        public int MainWindowLeft
        {
            get { return _MainWindowLeft; }
            set { _MainWindowLeft = value; }
        }
        private int _MainWindowLeft = 0;

        /// <summary>
        /// The Window State of the Main Window
        /// </summary>
        public int MainWindowState
        {
            get { return _MainWindowState; }
            set { _MainWindowState = value; }
        }
        private int _MainWindowState = 2;

        /// <summary>
        /// The Top of the Main Window
        /// </summary>
        public int MainWindowTop
        {
            get { return _MainWindowTop; }
            set { _MainWindowTop = value; }
        }
        private int _MainWindowTop = 0;

        /// <summary>
        /// The Width of the Main Window
        /// </summary>
        public int MainWindowWidth
        {
            get { return _MainWindowWidth; }
            set { _MainWindowWidth = value; }
        }
        private int _MainWindowWidth = 1225;

        #endregion Trigger Panel Properties


    }
}
