﻿//**************************************************************
// AppliedLogix DotNet Utilities Library
// Application Utilities class
//
// Copyright(c) 2014-2016 AppliedLogix, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//**************************************************************

using System;
using System.Windows;

namespace FliUtils.Mvvm
{
    /// <summary>
    /// The MvvmUtil class contains static methods useful
    /// in WPF MVVM and Commandline applications
    /// </summary>
    public class MvvmUtils
    {
        /// <summary>
        /// Run an action on the UI thread if this is a WPF app and
        /// we are not currently on the UI thread.
        /// Else, just run the action directly
        /// </summary>
        /// <param name="cmd">Action to run</param>
        public static void Invoke(Action cmd)
        {
            var app = Application.Current;
            if((app == null) || (app.Dispatcher.CheckAccess()))
            {
                cmd();
            }
            else
            {
                app.Dispatcher.Invoke(cmd);
            }
        }
    }
}
