﻿//**************************************************************
// AppliedLogix DotNet Utilities Library
// USB Device Connection/Disconnection Event and Information Class
//
// Copyright(c) 2016 AppliedLogix, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//**************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Management;
using System.Text;
using System.Threading.Tasks;

namespace FliUtils.Usb
{
    /// <summary>
    /// USB Device Identifier struct
    /// The USBMonitor DeviceID property is a collection of USBDeviceID structs.
    /// When the monitor detects a USB device has been connected or disconnected,
    /// It compares the devices ID parameters to the definitions in DeviceID.
    /// If a match is found, the corresponding event is raised.
    /// The USBDeviceInfo instance passed to the user's event handler includes the
    /// Key of the matching USBDeviceID instance.
    ///
    /// All empty string values are ignored in the comparison process.
    /// If the DeviceID collection is empty, events will be raised for all USB devices.
    /// </summary>
    public struct USBDeviceID
    {
        public string KEY;  // User defined tag identifying this instance
        public string VID;  // Vendor ID, 4 hex digits or empty
        public string PID;  // Product ID, 4 hex digits or empty
        public string MI;   // Multiple Interface Identifer, 2 hex digits or empty

        public USBDeviceID(string key, string vid = "", string pid = "", string mi = "")
        {
            KEY = key;
            VID = vid;
            PID = pid;
            MI = mi;
        }
        
        public override string ToString()
        {
            string str = "KEY '" + KEY + "'";
            str += ", VID " + (String.IsNullOrEmpty(VID)?"*":VID);
            str += ", PID " + (String.IsNullOrEmpty(PID)?"*":PID);
            str += ", MI "  + (String.IsNullOrEmpty(MI)?"*":MI);
            return str;
        }
    }
}

